import java.io.*;
import java.util.*;

/**
 *@author Jorma Sainio
 * A sequence solver to take part in Ohjelmointiputka's 
 * 'Intelligence' programming competition.
 * http://www.ohjelmointiputka.net/kilpa.php?tunnus=alyo
 *
 * Solves sequences by splitting and/or simplifying them
 * and then calling itself recursively again with the simpler
 * sequences. It usually ends with a numerical q[n]=a*q[n-1]+b
 * solver for numbers or single characters, but it can find a 
 * solution earlier if it detects a loop in the sequence.
 *
 * Because of an unfortunate incident, I managed to permanently 
 * delete my code for this project just few days before the deadline
 * and had recode the whole thing in about 5 days. Fortunately I 
 * managed to reimplement all features that were in the original
 * code and managed to add some extra bits. Unfortunately, because of 
 * the time limits, the code is not quite as clean and well commented
 * as it could be.
 */
public class AIQ {
  private static final boolean debug = false;
  public  static final String name = "aiq";

  private static final int maxterms = 10;
  private static final int maxtermlength = 20;
  
  /** 
   * Does char calculations wrap around from 
   * Z to A.
   */
  private static boolean bCharMod26 = true;

  /** debug level, for printing debug trees */
  private static int dl = 0;

  private static int diffLevel = 0;
  private static int stringSolverLevel = 0;
  private static int stringRecLevel = 0;
  private static int termRepeatLevel = 0;
  private static int flatListLevel = 0;
  private static int charwiseLevel = 0;
  private static int intSolverLevel = 0;
  private static int splitLevel = 0;
  
  private static FileOutputStream fos;

  /**
   * Prints debug information if debug is true.
   */
  private static final void debug(String s) {
    if (debug) {
      for (int i = 0; i < dl; i++)
        System.out.print("#");
      System.out.println(" "+s);
    }
  }
  
  /**
   * Throws exception if b is false
   */
  private static final void ASSERT(boolean b) {
    if (!b) throw new RuntimeException();
  }
  
  /**
   * Prints single solution term to file
   */
  private static final void printSol(String s) {
    debug(s);
    try {
      fos.write(s.getBytes());
      fos.write('\n');
    }catch(IOException ex) {
      ex.printStackTrace();
    }
  }

  /**
   * Even-odd split for int values.
   */
  private static final int[] intSplitEvenOdd(int [] q, int n) {
    if (q.length < 6)
      return null;
    debug("int even odd split");
    splitLevel ++;
    if (debug) dl++;
    try {
      int len = q.length;
      int [] even = new int[len/2+(len&0x1)];
      int [] odd  = new int[len/2];
      for (int i = 0; i < len; i++)
        if ( (i & 0x1) > 0)
          odd[i/2] = q[i];
        else 
          even[i/2] = q[i];
      
      int en = n/2;
      int on = n/2;
      boolean bOddFirst = false;
      if ((len &0x1) > 0)
        bOddFirst = true;
      if ( (n &0x1) > 0) {
        if (bOddFirst)
          on++;
        else
          en++;
      }
            
      int [] eres = null;
      if (en > 0) {
        eres = simpleIntSolver(even, en);
        if (eres == null)
          return null;
        debug("= even terms");
      }
      int [] ores = null;
      if (on > 0) {
        ores = simpleIntSolver(odd, on);
        if (ores == null)
          return null;
        debug("= odd terms");
      }
      int c = 0;
      int oc= 0;
      int ec = 0;
      int [] res = new int[n];
      if (bOddFirst) {
        res[c++] = ores[oc++];
      }
      while(c < n) {
        res[c++] = eres[ec++];
        if (c < n)
          res[c++] = ores[oc++];
      }
      
      debug("< even odd split");
      return res;
    
    }finally {
      if (debug) dl--;
      splitLevel --;
    }
  }

  /**
   * q[n] = a*q[n-1]+b
   * @return int[2]{a,b}
   */
  private static final int[] simpleRecSol(final int[] q) {
    int a = 0;
    int b = 0;
    if (q.length >= 3) {
      int nom = q[1]-q[2];
      int den = q[0]-q[1];
      if (nom == 0 || den == 0) {
        a = 1;
        b = 0;
      }
      else {
        if (nom % den != 0)
          return null;
      
        a = nom / den;
        b = q[1]-a*q[0];
      }
      for (int i = 1; i < q.length; i++) {
        if (q[i] != a*q[i-1]+b)
          return null;
      }

    }
    else if (q.length == 2 && diffLevel <= 1) {
      a = 1;
      b = q[1]-q[0];
    }
    else if (q.length == 1 && diffLevel == 0) {
      a = 1;
      b = 0;
    }
    else
      return null;
    
    debug("< q[n] = "+a+"*q[n-1] + "+b);
    
    return new int[] {a, b};
    
  }

  /**
   * This solves most of the integer problems.
   * q[n] = a*q[n-1]+b
   */
  private static final int[] simpleRecursive(final int [] q, int n) {
    debug("simple recursive");
    if (debug) dl++;
    try {

      int [] sol = simpleRecSol(q);
      if (sol == null)
        return null;
      int a = sol[0];
      int b = sol[1];
      
      int [] res = new int[n];
      int c =q[q.length-1];
      for (int i = 0; i < n; i++) {
        res[i] = a*c+b;
        c = res[i];
      }
      debug("< simple recursive");
      return res;
    }
    finally{
      if (debug) dl--;
    }
  }
  
  /**
   * Loop detection for strings.
   */
  private static final String[] loopFinder(final String[] q, int n) {
    if (q.length < 3)
      return null;
    debug("loopFinder");
    if (debug) dl++;
    try {
      int loop = -1;
      for (int i = 2; i <= (q.length+1)/2; i++) {
        boolean bSolution = true;
        for (int j = 0; j < q.length-i; j++) {
          if (!q[j].equals( q[j+i])) {
            bSolution = false;
            break;
          }
        }
        if (bSolution) {
          loop = i;
          break;
        }
      }
      if (loop == -1)
        return null;
      
      String[] res = new String[n];
      for (int i = 0; i < n; i++) {
        int x = i-loop;
        if (x < 0)
          res[i] = q[q.length+x];
        else
          res[i] = res[x];
        
      }
      debug("< string loop, length "+loop);
      return res;
    }
    finally{
      if (debug) dl--;
    }
  }

  /**
   * loop detecting for int values.
   */
  private static final int[] intLoop(final int[]q, int n) {
    if (q.length < 3)
      return null;
    debug("intLoop");
    if (debug) dl++;
    try {
      int loop = -1;
      for (int i = 2; i <= (q.length+1)/2; i++) {
        boolean bSolution = true;
        for (int j = 0; j < q.length-i; j++) {
          if (q[j] != q[j+i]) {
            bSolution = false;
            break;
          }
        }
        if (bSolution) {
          loop = i;
          break;
        }
      }
      if (loop == -1)
        return null;
      
      int[] res = new int[n];
      for (int i = 0; i < n; i++) {
        int x = i-loop;
        if (x < 0)
          res[i] = q[q.length+x];
        else
          res[i] = res[x];
        
      }
      debug("< int loop, length "+loop);
      return res;
    }
    finally{
      if (debug) dl--;
    }
  }

  /**
   * Reversed recursive series.
   * q[n] = (q[n-1]-b)/a
   */
  private static final int[] simpleRevRec(final int [] q, int n) {
    if (q.length < 3)
      return null;
    debug("simple rev rec");
    if (debug) dl ++;
    try {
      int nom = q[0]-q[1];
      int den = q[1]-q[2];
      if (nom == 0 || den == 0 || nom % den != 0)
        return null;
      int a = nom/den;
      int b = q[0]-a*q[1]; 
      
      //check
      for (int i = 1; i < q.length; i++) {
        int x = (q[i-1]-b);
        if (x % a != 0)
          return null;
        if (q[i] != x / a)
          return null;
      }
      
      int c = q[q.length-1];
      int [] res = new int[n];
      for (int i = 0; i < n; i++) {
        int x = c-b;
        if (x % a != 0)
          return null;
        res[i] = x / a;
        c = res[i];
      }
      debug("< simple rev rec");
      
      return res;
    }
    finally {
      if (debug) dl--;
    }
  }

  
  /**
   * Calculates the differencies of items and then
   * solve this difference queue.
   * q[n] = a[n]+q[n-1];
   * solve a.
   */
  private static final int[] diffSolver(final int [] q, int n) {
    if (q.length < 3)
      return null;
    debug("diff solver");
    
    if (debug) dl++;
    diffLevel++;
    try {
      int len_1= q.length -1;
      int [] diff = new int[len_1];

      for (int i = 0; i < len_1; i++) {
        diff[i] = q[i+1]-q[i];
      }
      int [] dres = basicIntSolver(diff, n);
      if (dres == null)
        return null;
      
      int [] res = new int[n];
      int c = q[len_1];
      for (int i = 0; i < n; i++) {
        res[i] = c + dres[i];
        c = res[i];
      }
      debug("< diff solver");
      return res;
      
    }
    finally{
      diffLevel--;
      if (debug) dl--;
    }
  }

  
  /**
   * Finds a simple solution that works for the first 3
   * items. Then check if the same rule works for the whole
   * apart from few items.
   * 7,70,700,7000,8,80,800 -> solution: q[n]= 10*q[n-1]
   *                        -> solution length = 4
   *                        -> solution base values 7,8 -> solver 
   */
  private static final int[] partialSolutionMatcher(final int[] q, int n) {
    if (q.length < 5)
      return null;

    debug("partial solution matcher");
    if (debug) dl++;
    try {
      int [] test = new int[3];
      for (int i = 0; i < 3; i++)
        test[i] = q[i];

      int [] sol = simpleRecSol(test);
      if (sol == null)
        return null;
      
      int a = sol[0];
      int b = sol[1];
      Vector vBase = new Vector();
      vBase.addElement(new Integer(q[0]));
      int solLength = -1;
      int lastBase = 0;
      for (int i = 1; i < q.length; i++) {
        if (q[i] != a*q[i-1]+b) {
          if (solLength == -1)
            solLength = i;
          else if (solLength != i-lastBase)
            return null;
          lastBase = i;
            
          vBase.addElement(new Integer(q[i]));
        }
      }
      if (solLength == q.length-1)
        return null;

      int d = -q.length % solLength;
      if (d < 0) d+=solLength;

      int [] ibase = new int[vBase.size()];
      for (int i = 0; i < ibase.length; i++) {
        ibase[i] = ((Integer)vBase.elementAt(i)).intValue();
      }

      int sum = d;
      int m = 0;
      for (int i = 0; i < n; i++, m++) {
        if (sum >= n)
          break;
      }
      
      int [] ires = null;
      if (m > 0) {
        ires = simpleIntSolver(ibase, m);
        if (ires == null)
          return null;
      }
      int c = q[q.length-1];
      int [] res = new int[n];
      int rc = 0;
      for (int i = 0; i < d; i++) {
        res[rc] = a*c+b;
        c = res[rc++];
        if (rc >= n)
          break;
      }
      
      if (rc < n) {
        solution:
        for (int i = 0; i < ires.length; i++) {
          res[rc] = ires[i];
          c = res[rc++];
          if (rc >= n)
            break;
          for (int j = 0; j < solLength; j++) {
            res[rc] = a*c+b;
            c = res[rc++];
            if (rc >= n)
              break solution;
          }
        }
      }

      debug("< partial solution");
      return res;
    }
    finally{
    
      if (debug) dl--;
    }
  }

  
  /**
   * q[n] = a*q[n-2]+b*a[n-1]
   * fibonacci etc.
   */
  private static final int[] linear(final int[] q, int n) {
    if (q.length < 5)
      return null;
    debug("linear search");
    if (debug) dl++;
    try {
      int nom = q[2]*q[2]-q[3]*q[1];
      int den = q[0]*q[2]-q[1]*q[1];
      if (den == 0 || nom % den != 0)
        return null;
      int a = nom / den;
      int b = 0;
      if (q[1] != 0) {
        b = q[2]-a*q[0];
        if (b % q[1] != 0)
          return null;
        b = b / q[1];
      }
      else if (q[2] != 0) {
        b = q[3]-a*q[1];
        if (b % q[2] != 0)
          return null;
        b = b / q[2];
      }
      else return null;
      
      for (int i = 2; i < q.length; i++) {
        if (q[i] != a*q[i-2]+b*q[i-1])
          return null;
      }
      int q1 = q[q.length-1];
      int q2 = q[q.length-2];
      int [] res = new int[n];
      for (int i = 0; i < n; i++) {
        res[i] = a*q2+b*q1;
        q2 = q1;
        q1 = res[i];
      }
      debug("< linear");
      return res;
      
    }
    finally{
      if (debug) dl--;
    }
  }

  /**
   * q[n] = a[n]*q[n-1]
   * solve a.
   * factorial etc.
   */
  private static final int[] mul(final int[] q, int n) {
    debug("mul solver");
    if (debug) dl++;
    try {
      
      int [] mul = new int[q.length-1];
      for (int i = 1; i < q.length; i++) {
        int a = q[i];
        int b = q[i-1];
        if (b == 0 || a % b != 0)
          return null;
        mul[i-1] = a / b;
      }
      int [] ires = basicIntSolver(mul, n);
      if (ires == null)
        return null;
      
      int [] res = new int[n];
      int c = q[q.length-1];
      for (int i = 0; i < n; i++) {
        res[i] = ires[i]*c;
        c = res[i];
      }
      
      debug("< mul");
      return res;
      
    }
    finally{
      if (debug) dl--;
    }
  }
  
  /**
   * q[n] = q[n-1]/a[n]
   * solve a. Only accurate integer division accepted.
   */
  private static final int[] revMul(final int [] q, int n) {
    debug("rev mul solver");
    if (debug) dl++;
    try {
      
      int [] mul = new int[q.length-1];
      for (int i = 1; i < q.length; i++) {
        int a = q[i-1];
        int b = q[i];
        if (b == 0 || a % b != 0)
          return null;
        mul[i-1] = a / b;
      }
      int [] ires = basicIntSolver(mul, n);
      if (ires == null)
        return null;
      
      int [] res = new int[n];
      int c = q[q.length-1];
      for (int i = 0; i < n; i++) {
        if ( ires[i] == 0 || c % ires[i] != 0)
          return null;
        res[i] = c / ires[i];
        c = res[i];
      }
      
      debug("< rev mul");
      return res;
      
    }
    finally{
      if (debug) dl--;
    }
  }
  
  
  /**
   * Simpler version of the basicIntSolver.
   * Less probable of giving 'wrong' solution, but
   * also less powerfull.
   */
  private static final int[] simpleIntSolver(final int [] q, int n) {
    debug("simple int solver");
    if (debug) dl++;
    try {
      int [] res = null;
      
      res = intLoop(q, n);
      if (res == null)
        res = simpleRecursive(q,n);
      
      if (res == null)
        res = simpleRevRec(q,n);

      //partial solution matcher?
      if (res != null)
        debug("< simple int solver");
      
      return res;
    }
    finally {
      if (debug) dl--;
    }
  }
   
  /**
   * Solver integer queues. 
   */
  private static final int[] basicIntSolver(final int [] q, int n) {
    debug("basic int solver");
    intSolverLevel ++;
    if (debug) dl++;
    try {
      int [] res = null;
    
      res = intLoop(q, n);
      if (res == null)
        res = simpleRecursive(q,n);
      if (res == null)
        res = simpleRevRec(q,n);
      
      if (res == null 
          && diffLevel < 3 
          && splitLevel == 0
          && (flatListLevel == 0 || diffLevel <1)
          && (charwiseLevel == 0 || diffLevel <1))
        res = diffSolver(q, n);
      
      if (res == null && intSolverLevel > 1)
        return null;
      
      if (res == null && diffLevel == 0)
        res = partialSolutionMatcher(q, n);
      
      if (res == null)
        res = linear(q, n);

      if (res == null)
        res = mul(q, n);
      
      if (res == null)
        res = revMul(q, n);
      if (res == null && diffLevel == 0)
        res = intSplitEvenOdd(q, n);
      if (res != null)
        debug("< basic int solver");
      
      return res;
    }
    finally {
      intSolverLevel --;
      if (debug) dl--;
    }
  }
  
  /**
   * Single term repeater. No real use.
   * QWERTY -> QWERTY,QWERTY,...
   */
  private static final String[] repeater(String[] q, int n) {
    if (q.length != 1)
      return null;
    String[] res = new String[n];
    for (int i = 0; i < n; i++)
      res[i] = q[0];
    
    debug("< repeater");
    return res;
  }

  /**
   * A, BB, BB, CCC, CCC, CCC -> A,BB,CCC
   *           repetitions    -> 1,2,3
   */
  private static final String[] termRepeat(String[] q, int n) {
    if (termRepeatLevel > 0)
      return null;
    debug("term repeat");
    termRepeatLevel ++;
    if (debug) dl++;
    try {
      Vector vTerms = new Vector();
      Vector vCount = new Vector();
      int iPacked = 0;
      int c = 0;
      String prev = null;
      for (int i = 0; i < q.length; i++) {
        String s = q[i];
        if (s.equals(prev)) {
          c++;
        }
        else {
          if (c > 0) {
            if (c > 1) iPacked ++;
            vCount.addElement(new Integer(c));
            vTerms.addElement(prev);
          }
          
          c = 1;
          prev = s;
        }

      }
      if (c > 0) {
        if (c > 1) iPacked++;
        vCount.addElement(new Integer(c));
        vTerms.addElement(prev);
      }
      
      if (iPacked < 2)
        return null;

      int [] reps = new int[vCount.size()];
      for (int i = 0; i < reps.length; i++) {
        reps[i] = ((Integer)vCount.elementAt(i)).intValue();
      }
      
      int [] ires = simpleIntSolver(reps, n);
      if (ires == null)
        return null;
      debug("= repeat pattern");

      int m = -1;
      int sum = 0;
      for (int i = 0; i < ires.length; i++) {
        sum += ires[i];
        if (sum >= n) {
          m = i+1;
          break;
        }
        if (ires[i] <= 0)
          return null;
      }
      if (m < 0)
        return null;
      
      String [] terms = (String[])vTerms.toArray(new String[0]);
      
      String [] sres = stringSolver(terms, m);
      if (sres == null)
        return null;
      debug("= next terms");
            
      String [] res = new String[n];
      c = 0;
      solution:
      for (int i = 0; i < sres.length; i++) {
        int repeat = ires[i];
        for (int j = 0; j < repeat; j++) {
          res[c++] = sres[i];
          if (c >= n)
            break solution;
        }
      }
      debug("< term repeat");
      return res;

    }finally {
      if (debug) dl--;
      termRepeatLevel --;
    }
  }

  /**
   * Finds the pattern and number of repetitions in each queue item.
   * AA, ABAB, ABCABC -> A,AB,ABC
   *                  -> 2,2,2
   */
  private static final String[] patternRepeat(String [] q, int n) {
    if (q.length < 3)
      return null;
    debug("patternRepeat");
    if (debug) dl++;
    try {
      String[] pattern = new String[q.length];
      int[] reps = new int[q.length];
      for (int i = 0; i < q.length; i++) {
        int len = q[i].length();
        boolean bFail = true;
        for (int ll = 1; 2*ll <= len; ll++) {
          if (len % ll != 0) //TODO accept same remainder from all
            continue;

          boolean bSolution = true;
          for(int j = 0; j < len-ll;j++) {
            if (q[i].charAt(j) != q[i].charAt(j+ll)) {
              bSolution = false;
              break;
            }
          }
          if (bSolution) {
            pattern[i] = q[i].substring(0, ll);
            reps[i] = len / ll;
            bFail = false;
            break;
          }
        }
        if (bFail) 
          return null;
      }

      int [] ires = basicIntSolver(reps, n);
      if (ires == null)
        return null;

      String [] sres = stringSolver(pattern, n);
      if (sres == null)
        return null;

      String [] res = new String[n];
      for (int i = 0;i < n; i++) {
        StringBuffer sb = new StringBuffer();
        for (int j = 0; j < ires[i]; j++)
          sb.append(sres[i]);
        res[i] = sb.toString();
      }
      
      debug("< pattern repeat");
      return res;
    }finally {
      if (debug) dl--;
    }
  }
  
  /**
   * Handles queues with possible empty values.
   */
  private static final String[] nullStringSolver(String[] q, int n) {
    Vector vEmpty = new Vector();
    Vector vTerms = new Vector();
    for (int i = 0; i < q.length; i++) {
      if (q[i] == null || q[i].equals(""))
        vEmpty.addElement(new Integer(i));
      else
        vTerms.addElement(q[i]);
    }

    if (vEmpty.size() == q.length) {
      String[] res = new String[n];
      for (int i = 0; i < n; i++)
        res[i] = "";
      debug("< null string solver (empty queue)");
      return res;
    }
    if (vEmpty.size() == 0)
      return stringSolver(q, n);

    int[] ies = new int[vEmpty.size()];
    for (int i = 0; i < ies.length; i++)
      ies[i] = ((Integer)vEmpty.elementAt(i)).intValue();
    
    int [] ires = simpleIntSolver(ies, n);
    if (ires == null)
      return null;
    if (ires[0] < q.length)
      return null;
    for (int i = 1; i < n; i++)
      if (ires[i] < q.length+i || ires[i] <= ires[i-1])
        return null;
    
    int m = 0;
    int c = 0;
    int len = q.length;
    for (int i = 0; i < n; i++)
      if (i+len == ires[c])
        c++;
      else
        m++;
    if (m == 0) //too bad soluttion
      return null;
    
    String [] sTerms = (String[])vTerms.toArray(new String[0]);
    


    String[] sres = stringSolver(sTerms, m);
    if (sres == null)
      return null;
    
    c = 0;
    m = 0;
    String[] res = new String[n];
    for (int i = 0; i < n; i++) {
      if (i+len == ires[c]) {
        res[i] = "";
        c++;
      }
      else {
        res[i] = sres[m++];
      }
    }
    debug("< nullString solver");
    return res;
  }

  private static final String[] stringRecursiveCore(String[] q,
                                                    String[] before, 
                                                    String[] after, int n) {
    debug("string recursive core");
    if (debug) dl++;
    try {
      
      //combine before and after difference
      String [] comb = new String[before.length];
      int[] lbefore = new int[before.length];
      int[] lafter  = new int[before.length];
      for (int i = 0; i < before.length; i++) {
        lbefore[i] = before[i].length();
        lafter[i] = after[i].length();
        comb[i] = before[i]+after[i];
      }
      
      int [] lbres = basicIntSolver(lbefore, n);
      if (lbres != null) {
        int[] lares = basicIntSolver(lafter, n);
        if (lares != null) {
          String[] sres = nullStringSolver(comb, n);
          if (sres != null) {
            String s = q[q.length-1];
            String [] res = new String[n];
            for (int i = 0; i < n; i++) {
              if (lbres[i] < 0 || lares[i] < 0)
                return null;
              if (lbres[i] == 0)
                res[i] = s+sres[i];
              else if (lares[i] == 0)
                res[i] = sres[i]+s;
              else {
                if (sres[i].length() != lbres[i]+lares[i])
                  return null;
                res[i] = sres[i].substring(0, lbres[i])
                  + s + sres[i].substring(lbres[i]);
              }
              s = res[i];
            }
            debug("< str rec core (comb)");
            return res;
          }
        }
      }
      
      //solve before and after difference separedly
      String [] sbres = nullStringSolver(before, n);
      if (sbres == null)
        return null;
      
      String [] sares = nullStringSolver(after, n);
      if (sares == null)
        return null;

      String s = q[q.length-1];
      String [] res = new String[n];
      for (int i = 0; i < n; i++) {
        res[i] = sbres[i] + s + sares[i];
        s = res[i];
      }
      debug("< string rec core (sep)");
      return res;
    }finally {
      if (debug) dl--;
    }
  }

  /**
   * q[n] = b[n] | q[n-1] | a[n]
   * solve b, a 
   */
  private static final String[] stringRecursive(String [] q, int n) {
    if (q.length < 3 || stringRecLevel != 0)
      return null;
    stringRecLevel ++;
    debug("stringRecursive");
    if (debug) dl++;
    try {
      
      String[] before = new String[q.length-1];
      String[] after = new String[q.length-1];
      
      for (int i = 1; i < q.length; i++) {
        int ix = q[i].indexOf(q[i-1]);
        if (ix == -1)
          return null;
        before[i-1] = q[i].substring(0, ix);
        after[i-1] = q[i].substring(ix+q[i-1].length());
      }
      
      {
        //add the first term to the difference queues if possible
        boolean bBeforeEmpty = true;
        boolean bBeforeHasEmpty = false;
        boolean bAfterEmpty = true;
        boolean bAfterHasEmpty = false;
        
        for (int i = 0; i < before.length; i++) {
          if (before[i].length() == 0)
            bBeforeHasEmpty = true;
          else
            bBeforeEmpty = false;

          if (after[i].length() == 0)
            bAfterHasEmpty = true;
          else
            bAfterEmpty = false;
        }
        boolean bBeforeAdded = false;
        boolean bAfterAdded = false;
        
        if (!bBeforeEmpty 
            && !bBeforeHasEmpty
            && (q.length > 3 
                || 
                (before[0].length() == before[1].length()
                 && q[0].length() == before[0].length())))
          bBeforeAdded = true;
        if (!bAfterEmpty 
            && !bAfterHasEmpty
            && (q.length > 3 
                || 
                (after[0].length() == after[1].length()
                 && q[0].length() == after[0].length())))
          bAfterAdded = true;
        
        if ((bBeforeAdded || bBeforeEmpty)
          && (bAfterAdded || bAfterEmpty)) {
          
          String [] addbef = new String[q.length];
          if (bBeforeEmpty)
            addbef[0] = "";
          else
            addbef[0] = q[0];
          String [] addaft = new String[q.length];
          if (bAfterEmpty)
            addaft[0] = "";
          else
            addaft[0] = q[0];
          
          for (int i = 1; i < q.length; i++) {
            addbef[i] = before[i-1];
            addaft[i] = after[i-1];
          }
          String [] res = stringRecursiveCore(q, addbef, addaft, n);
          if (res != null) {
            debug("< string recursive (with first term)");
            return res;
          }
        }
        
      }

      String [] res = stringRecursiveCore(q, before, after, n);
      
         
      if (res != null)
        debug("< string recursive");
      return res;
    }finally {
      stringRecLevel --;
      if (debug) dl--;
    }
  }
  
  /**
   * q[n-1] = XXX | q[n] | YYY
   * Solve XXX.lengths and q[n].lengths.
   */
  private static final String[] revStringRecursive(String [] q, int n) {
    if (q.length < 3)
      return null;

    debug("revStringRecursive");
    if (debug) dl++;
    try {
      int [] lens = new int[q.length];
      int [] cut = new int[q.length-1];
      lens[0] = q[0].length();
      for (int i = 1; i < q.length; i++) {
        int ix  = q[i-1].indexOf(q[i]);
        if (ix == -1)
          return null;
        lens[i] = q[i].length();
        cut[i-1] = ix;
      }
      int [] lres = basicIntSolver(lens, n);
      if (lres == null)
        return null;
      int [] cres = basicIntSolver(cut, n);
      if (cres == null)
        return null;
      String s = q[q.length-1];
      String [] res = new String[n];
      for (int i = 0; i < n; i++) {
        int a = cres[i];
        int l = lres[i];
        int len = s.length();
        if (a < 0 || l < 0 || a >= len || a+l > len)
          return null;
        res[i] = s.substring(a, a+l);
        s = res[i];
      }
      debug("< rev string recursive");
      return res;
    }finally {
      if (debug) dl--;
    }
  }
  
  /**
   * Solves a normal char palindrome.
   * A, ABA, ABCBA -> A,AB,ABC
   *      lengths  -> 1, 3, 5 
   */
  private static final String[] palindrome(String [] q, int n) {
    if (q.length < 3)
      return null;
    
    debug("palindrome");
    if (debug) dl++;
    try {
      int[] lens = new int[q.length];
      String [] sq = new String[q.length];
      int iContinue = 0;
      for (int i = 0; i < q.length; i++) {
        lens[i] = q[i].length();
        if (lens[i] > 1)
          iContinue ++;
        int x = lens[i] / 2;
        
        for (int j = 0; j < x; j++) {
          if (q[i].charAt(j) != q[i].charAt(lens[i]-1-j))
            return null;
        }
        sq[i] = q[i].substring(0, x+(lens[i]&0x1));
      }

      if (iContinue < 2)
        return null;

      int [] ires = basicIntSolver(lens, n);
      if (ires == null)
        return null;
      
      String[] sres = stringSolver(sq, n);
      if (sres == null)
        return null;

      String [] res = new String[n];
      
      for (int i = 0; i < n; i++) {
        if ( (ires[i] & 0x1) > 0) {
          if (ires[i] != 2*sres[i].length()-1)
            return null;
          res[i] = sres[i]+reverse(sres[i]).substring(1);
        }
        else {
          if (ires[i] != 2*sres[i].length())
            return null;
          res[i] = sres[i]+reverse(sres[i]);
        }
      }

      debug("< char palindrome");
      return res;
    }finally {
      if (debug) dl--;
    }
  }

  /**
   * converts a numberpalindrome to charpalindrome. Returns
   * also a type mask that say which chars were originally
   * numbers. 
   * @return String[2] {modified palindrome, char/int pattern(A|B)*}
   */
  private static final String[] number2charPalindrome(String s) {
    int len = s.length();
    if (len == 0)
      return new String[] {"",""};
    
    //for (int i = 0; i < len; i++) {
    char c = s.charAt(0);
    char t;
    if (c <= '9' && c >= '0')
      t = 'B';
    else 
      t = 'A';

    int ix = len;
    while (true) {
      ix = s.lastIndexOf(c, ix-1);
      if (ix == 0) {
        if (t == 'A' && len != 1)
          return null;
        else if (t == 'B') {
          try {
            s = ""+(char)('A'+Integer.parseInt(s));
          }catch(NumberFormatException ex) {
            return null;
          }
        }
        return new String[] {s, ""+t};
      }
      
      int x = len - ix;
      if (x > 1 && t == 'A' || x > ix)
        return null;
        
      String cut = s.substring(0,x);
      if (t == 'B') {
        try {
          cut = ""+(char)('A'+Integer.parseInt(cut));
        }catch(NumberFormatException ex) {
          return null;
        }
      }
      
      String [] a = number2charPalindrome(s.substring(x, ix));
      if (a != null) {
        return new String[] {cut+a[0]+cut, t+a[1]+t};
      }
    }
    
  }

  /**
   * Converts charPalindrome to numberpalindrome.
   * @param s charpalindrome
   * @param type type mask ('A' = char, 'B' = char)
   */
  private static final String char2numberPalindrome(String s, String type) {
    int len = s.length();
    StringBuffer sb = new StringBuffer();
    for (int i = 0; i < len; i++) {
      char t  = type.charAt(i);
      if (t == 'A')
        sb.append(s.charAt(i));
      else if (t == 'B')
        sb.append((int)(s.charAt(i)-'A'));
      else
        return null;
    }
    return sb.toString();
  }

  /**
   * Helper function for numberPalindrome
   */
  private static final String[] numberPalindromeCore(String[] charPalindrome, 
                                                     String[] types, int n) {
    String [] tres = palindrome(types, n);
    if (tres == null)
      return null;
    String [] cres = palindrome(charPalindrome, n);
    if (cres == null)
      return null;
    

    String [] res = new String[n];
    for (int i = 0; i < n; i++) {
      String s = char2numberPalindrome(cres[i], tres[i]);
      if (s == null)
        return null;
      res[i] = s; 
    }
    return res;
  }

  /**
   * Solves a palindrome where the where 'chars' are numbers.
   * 101110, 1011121110, 10111213121110...
   */
  private static final String[] numberPalindrome(String [] q, int n) {
    if (q.length < 3)
      return null;
    debug("number palindrome");
    if (debug) dl++;
    try {
      String [] cq = new String[q.length];
      String [] type = new String[q.length];
      
      for (int i = 0; i < q.length; i++) {
        String[] a = number2charPalindrome(q[i]);
        if (a == null)
          return null;
        cq[i] = a[0];

        type[i] = a[1]; 
      }
      
      String[] res = numberPalindromeCore(cq, type, n);
      
      if (res == null) {
        //fix BB (11) to ('A'+11)
        for (int i = 0; i < q.length; i++) {
          int len = cq[i].length();
          char p = cq[i].charAt(0);
          for (int j = 1; j < len; j++) {
            char nc = cq[i].charAt(j);
            if (p == nc
                && type[i].charAt(j)=='B'
                && type[i].charAt(j-1)=='B') {
              
              int x = p-'A';
              int k = 10;
              for(; k < x; k*=10);
              x = x*k+x;
                
              
              cq[i] = cq[i].substring(0,j-1)
                +((char)('A'+x))+cq[i].substring(j+1);
              
              type[i] = new StringBuffer(type[i]).deleteCharAt(j).toString();
              len--;
              j--;
              p = 0;
            }
            else
              p = nc;
           
          }
        }
        res = numberPalindromeCore(cq, type, n);
      }
        
      debug("< number palindrome");
      return res;
    }finally {
      if (debug) dl--;
    }
  }
  
  /**
   * Checks if all the items in the queue can be acquired
   * by shifting the first item. Then this shift length 
   * queue is solved with intSolver.
   * AB1C, B1CA, 1CAB -> -1, -2 = 3, 6 -> int solver
   */
  private static final String[] shiftSolver(String [] q, int n) {
    if (q.length < 2)
      return null;

    debug("shift");
    if (debug) dl++;
    try {
      int len = q[0].length();
      int [] shs = new int[q.length-1];
      for (int i = 1; i < q.length; i++) {
        if (q[i].length() != len)
          return null;
        boolean bSol = false;
        for (int s = 1; s < len; s++) {
          boolean bFail = false;
          for (int j = 0; j < len; j++) {
            if (q[0].charAt(j) != q[i].charAt((j+s)%len)) {
              bFail = true;
              break;
            }
          }
          if (!bFail) {
            bSol = true;
            shs[i-1] = s;
            break;
          }
        }
        if (!bSol)
          return null;
      }
      for (int i = 1; i < shs.length; i++)
        while(shs[i] < shs[i-1]) shs[i]+=len;
      
      int [] ires = basicIntSolver(shs, n);
      if (ires == null)
        return null;
      
      String[] res = new String[n];
      for (int i = 0; i < n; i++) {
        res[i] = shift(q[0], ires[i]);
      }
      debug("< shift solver");
      return res;
    }finally {
      if (debug) dl--;
    }
  }

  /**
   * Converts strings to ints and calls intsolver.
   * Also converts single characters to int.
   * Handles leading zeroes.
   */
  private static final String[] charAndIntSolver(String [] q, int n) {
    debug("char and int solver");
    if (debug) dl++;
    try {
      Vector vCharPattern = new Vector();
      boolean bLeadingZeroes = false;
      Vector vZeroLengths = new Vector();
      Vector vNumberLengths= new Vector();
      int [] iq = new int[q.length];
      for (int i = 0; i < q.length; i++) {
        int a = -1;
        int len = q[i].length();
        int c = q[i].charAt(0);
        try {
          if (c <= '9' && c >= '0') {
            a = Integer.parseInt(q[i]);
            
            
            vNumberLengths.addElement(new Integer(len));
            int zs = 0;
            if (c == '0') {
              zs = 1;
              for (int j = 1; j < len; j++, zs++) {
                if (q[i].charAt(j) != '0')
                  break;
              }
              if (a == 0)
                zs--;
              if (zs > 0)
                bLeadingZeroes = true;
            }
            vZeroLengths.addElement(new Integer(zs));
          }
        }catch(NumberFormatException ex) {}
        if (a == -1) {
          if (len > 1)
            return null;
          vCharPattern.addElement(new Integer(i));
          iq[i] = (c-'A'+1);
        }
        else
          iq[i] = a;
      }
      
      boolean bRegularNumLength = false;
      int [] numLengthPattern = null;

      if (bLeadingZeroes) {

        int [] lens = new int[vNumberLengths.size()];
        for (int i = 0; i < lens.length; i++)
          lens[i] = ((Integer)vNumberLengths.elementAt(i)).intValue();
        
        
        debug("Number length check");
        int [] ir = simpleIntSolver(lens, n);

        if (ir != null) {
          bRegularNumLength = true;
          numLengthPattern = ir;
          debug("= number length pattern");
        }
        else {
          
          for (int i = 0; i < lens.length; i++)
            lens[i] = ((Integer)vZeroLengths.elementAt(i)).intValue();

          debug("Zero length check");
          ir = basicIntSolver(lens, n);
          if (ir == null)
            bLeadingZeroes = false; //maybe should return null;
          else
            debug("= zero length pattern");
          numLengthPattern = ir;
          
        }
        
        
      }
      
      int[] cp = null;
      int cs = vCharPattern.size(); 

      if (cs > 0) {
        if (cs == q.length) {
          cp = null;
          if (bCharMod26)
            for( int i = 1; i < iq.length; i++)
              while(iq[i] < iq[i-1]) iq[i] += 26;
          
        }
        else {
          int [] pattern = new int[cs];
          for (int i = 0; i < cs; i++) {
            pattern[i] = ((Integer)vCharPattern.elementAt(i)).intValue();
          }
          cp = basicIntSolver(pattern, n);
          if (cp == null)
            return null;

        }
      }
      else {
        cp = new int[1];
        cp[0] = 1000000;
      }

      int [] ires = basicIntSolver(iq, n);
      if (ires == null)
        return null;

      String [] res = new String[n];
      int nextp = 0;
      int nextn = 0;
      for (int i = 0; i < n; i++) {
        if (cp != null && cp[nextp] < q.length + i )
          return null;

        if (cp == null || cp[nextp] == q.length + i) {
          nextp++;
          int x = ires[i];
          if (bCharMod26) {
            while( x < 1) x+=26;
            x = (x-1)%26+1;
          }
          res[i] = ""+(char)(x-1+'A');
        }
        else {
          String s = ""+ires[i];
          if (bLeadingZeroes) {
            if (bRegularNumLength) {
              int nl = numLengthPattern[nextn++];
              int sl = s.length();
              for (int j = sl; j < nl; j++)
                s = "0"+s; //TODO to stringbuffer
            }
            else {
              int nl = numLengthPattern[nextn++];
              for (int j = 0; j < nl; j++)
                s = 0+s;
            }
            
          }
          res[i] = s;
        }
      }
      
      debug("< char and int solver");
      return res;
      
    }finally {
      if (debug) dl--;
    }
  }

  /**
   * AX, BY, CZ -> A,B,C
   *            -> X,Y,Z
   */
  private static final String[] charwiseSplit(String [] q, int n) {
    if (q.length < 2)
      return null;
    int len = q[0].length();
    if (len < 2)
      return null;
    debug("charwise split");
    charwiseLevel++;
    if (debug) dl++;
    try {
      String[][] splitQs = new String[len][q.length];
      for (int i = 0; i < q.length; i++) {
        if (q[i].length() != len)
          return null;
        for (int j = 0; j < len; j++)
          splitQs[j][i] = ""+q[i].charAt(j);
      }
      
      String[][] resQs = new String[len][];
      for (int i = 0; i < len; i++) {
        resQs[i] = stringSolver(splitQs[i], n);
        if (resQs[i] == null)
          return null;
      }

      String [] res = new String[n];
      for (int i = 0; i < n; i++) {
        StringBuffer sb = new StringBuffer();
        for (int j = 0; j < len; j++)
          sb.append(resQs[j][i]);
        res[i] = sb.toString();
      }
      
      debug("< charwise split");
      return res;
    }finally {
      if (debug) dl--;
      charwiseLevel--;
    }
  }
  
  /**
   * A10X, B9Y, C8Z -> A,B,C
   *                -> 10, 9, 8
   *                -> X,Y,Z
   */
  private static final String[] charIntSplit(String [] q, int n) {
    debug("char-int split");
    if (debug) dl++;
    try {
      
      String [][] splits = null;
      Vector vSplits = new Vector();
      int qs = -1;
      for (int i = 0; i < q.length; i++) {
        int len = q[i].length();
        vSplits.clear();
        int type = 0;
        StringBuffer sb = new StringBuffer();
        for (int j = 0; j < len; j++) {
          char c = q[i].charAt(j);
          if ( c >= '0' && c <= '9') {
            if (type == 2) {
              vSplits.addElement(sb.toString());
              sb = new StringBuffer();;
            }
            sb.append(c);
            type = 1;
          }
          else if( c >= 'A' && c <= 'Z') {
            if (type == 1) {
              vSplits.addElement(sb.toString());
              sb = new StringBuffer();;
            }
            sb.append(c);
            type = 2;
          }
          else
            return null;
        }
        if (type != 0) {
          vSplits.addElement(sb.toString());
        }

        if (qs == -1) {
          qs = vSplits.size();
          if (qs < 2)
            return null;
          splits = new String[qs][q.length];
        }
        
        if ( qs != vSplits.size() )
          return null;

        for (int j = 0; j < qs; j++) {
          splits[j][i] = (String)vSplits.elementAt(j);
        }
      }
      
      String [][] sres = new String[qs][];
      for (int i = 0; i < qs; i++) {
        sres[i] = stringSolver(splits[i], n);
        if (sres[i] == null)
          return null;
      }
      
      String [] res = new String[n];
      for (int i = 0; i < n; i++) {
        StringBuffer sb = new StringBuffer();
        for (int j = 0; j < qs; j++) {
          sb.append( sres[j][i] );
        }
        res[i] = sb.toString();
      }
      
      debug("< char int split");
      return res;
    }finally {
      if (debug) dl--;
    }
  }
  
  /**
   * A, AAB,AAABBC -> A,AB,ABC -> string solver
   * char lengths -> 1, (2,1), (3,2,1) -> B,CB,DCB -> string solver
   */
  private static final String[] charPacker(String [] q, int n) {
    debug("char packer");
    if (debug) dl++;
    try {
      String [] packed = new String[q.length];
      String [] count = new String[q.length];
      boolean bPacked = false;
      for (int i = 0; i < q.length; i++) {
        int len = q[i].length();
        StringBuffer sbPacked = new StringBuffer();
        StringBuffer sbCount = new StringBuffer();
        
        char prev = q[i].charAt(0) ;
        int cc = 1;        
        for (int j = 1; j < len; j++ ) {
          char c = q[i].charAt(j);
          if (c == prev) {
            cc++;
          }
          else {
            sbPacked.append( prev );
            sbCount.append( (char)('A'+cc) );
            if (cc > 1)
              bPacked = true;

            prev = c;
            cc = 1;
          }
        }
        sbPacked.append( prev );
        sbCount.append( (char)('A'+cc) );
        packed[i] = sbPacked.toString();
        count[i] = sbCount.toString();
        if (cc > 1)
          bPacked = true;
      }
      if (!bPacked)
        return null;

      
      String [] pres = stringSolver(packed, n);
      if (pres == null)
        return null;
      debug("= chars");

      boolean bOld = bCharMod26;
      bCharMod26 = false;
      String [] cres = null;
      try {
        cres = stringSolver(count, n);
      }finally {
        bCharMod26 = bOld;
      }
      if (cres == null)
        return null;
      debug("= char counts");

      String [] res = new String[n];
      for (int i = 0; i < n; i++) {
        String s = pres[i];
        String sCount = cres[i];
        int len = sCount.length();
        if (len != s.length())
          return null;
        StringBuffer sb = new StringBuffer();
        for (int j = 0; j < len; j++) {
          char c = pres[i].charAt(j);
          int x = (int)(sCount.charAt(j)-'A');
          for (int k = 0; k < x; k++)
            sb.append(c);
        }
        res[i] = sb.toString();
      }
      debug("< char packer");
      return res;
    }finally {
      if (debug) dl--;
    }
  }

  
  /**
   * A, BC, DEF -> A,B,C,D,E,F -> string solver
   *    lengths -> 1,2,3 -> int solver
   */
  private static final String[] flatlist(final String [] q, int n) {
    debug("flatlist");
    flatListLevel ++;
    if (debug) dl++;
    try {
      
      int [] lens = new int[q.length];
      boolean bContinue = false;
      int sum = 0;
      for (int i = 0; i < q.length; i++) {
        lens[i] = q[i].length();
        if (lens[i] > 1)
          bContinue = true;
        sum += lens[i];
      }
      if (!bContinue)
        return null;


      int [] lres = basicIntSolver(lens, n);
      if (lres == null)
        return null;
      debug("= flat list lengths");

      int results = 0; 
      for (int i = 0; i < lres.length; i++) {
        if (lres[i] <= 0 || lres[i] > maxtermlength)
          return null;
        results += lres[i];
      }
        
      String [] flat = new String[sum];
      int c = 0;
      for (int i = 0; i < q.length; i++) {
        int len = lens[i];
        for (int j = 0; j < len; j++) {
          flat[c++] = ""+q[i].charAt(j);     
        }
      }
      
      String [] sres = stringSolver(flat, results);
      if (sres == null)
        return null;

      debug("= flat list terms");
      c = 0;
      String [] res = new String[n];
      for (int i = 0; i < n; i++) {
        int x = lres[i];
        StringBuffer sb = new StringBuffer();
        for (int j = 0; j < x; j++) {
          sb.append(sres[c++]);
        }
        res[i] = sb.toString();
      }
      debug("< flatlist");
      return res;

    }finally {
      if (debug) dl--;
      flatListLevel --;
    }
  }
  
  
  /**
   * q[n] = q[n-2]|q[n-1]
   * or 
   * q[n] = q[n-1]|q[n-2]
   */
  private static final String[] stringfib(String [] q, int n) {
    if (q.length < 4)
      return null;
    debug("stringfib");
    if (debug) dl++;
    try {
      boolean bFail = false;
      for (int i = 2; i < q.length; i++) {
        String s = q[i-2]+q[i-1];
        if (!q[i].equals(s)) {
          bFail = true;
          break;
        }
      }
      if (!bFail) {
        
        String q1 = q[q.length-1];
        String q2 = q[q.length-2];
        String [] res = new String[n];
        for (int i = 0; i < n; i++) {
          res[i] = q2+q1;
          q2 = q1;
          q1 = res[i];
        }
        debug("< q[n] = q[n-2]|q[n-1]");
        return res;
      }

      for (int i = 2; i < q.length; i++) {
        String s = q[i-1]+q[i-2];
        if (!q[i].equals(s))
          return null;
      }
      String q1 = q[q.length-1];
      String q2 = q[q.length-2];
      String [] res = new String[n];
      for (int i = 0; i < n; i++) {
        res[i] = q1+q2;
        q2 = q1;
        q1 = res[i];
      }
      debug("< q[n] = q[n-1]|q[n-2]");
      return res;

    }finally {
      if (debug) dl--;
    }
  }
  
  /**
   * BAA, ABA, AAB, CAA,ACA,AAC  -> BAA, CAA
   *                      shifts -> ABC, ABC 
   */
  private static final String[] shiftPacker(String [] q, int n) {
    if (q.length < 4)
      return null;
    debug("shift packer");
    if (debug) dl++;
    try {
      
      int len = q[0].length();
      int [] shs = new int[q.length-1];
      
      Vector vBases = new Vector();
      Vector vCounts = new Vector();
      
      String sBase = q[0];
      String count = "A";
      int iPack = 0;
      int shiftCount = 0;

      for (int i = 1; i < q.length; i++) {
        if (q[i].length() != len) {
          vBases.addElement(sBase);
          vCounts.addElement(count);
          if (count.length() > 1)
            iPack++;
          count = "A";
          sBase = q[i];
          len = sBase.length();
          continue;
        }
        
        boolean bSol = false;
        for (int s = 0; s < len; s++) { 
          boolean bFail = false;
          for (int j = 0; j < len; j++) {
            if (sBase.charAt(j) != q[i].charAt((j+s)%len)) {
              bFail = true;
              break;
            }
          }
          if (!bFail) {
            bSol = true;
            if (s > 0)
              shiftCount++;
            int x = ('A'+s);
            int y = count.charAt(count.length()-1);
            while(x < y) x+= len;
            count +=  (char)x;     
            break;
          }
        }
        if (!bSol) {
          vBases.addElement(sBase);
          vCounts.addElement(count);
          if (count.length() > 1)
            iPack++;
          sBase = q[i];
          count = "A";
        }

      }
      vBases.addElement(sBase);
      vCounts.addElement(count);
      if (count.length() > 1)
        iPack++;

      if (2*iPack  < vBases.size() || shiftCount < 2 )
        return null;
      String [] counts = (String[])vCounts.toArray(new String[0]);
      String [] bases = (String[])vBases.toArray(new String[0]);      
      
      boolean bNegativeShifts = false;
      boolean bOld = bCharMod26;
      bCharMod26 = false;
      String[] cres = null;
      try {
        cres = stringSolver(counts, n);

        if (cres == null) { 
          bNegativeShifts = true;
          for (int i = 0; i < counts.length; i++) {
            int clen = counts[i].length();
            int baselen = bases[i].length();
            StringBuffer sb = new StringBuffer();
            int prev = 'A';
            for (int j = 0; j < clen; j++) {
              char c = counts[i].charAt(j);
              if (c == 'A') {
                sb.append(c);
                prev = c;
              }
              else {
                int x = ((c-'A') % baselen)-baselen;
                x = 'A'-x;
                while(x < prev) x+= baselen;
                sb.append((char)x);
                prev = x;
              }
            }
            counts[i] = sb.toString();
          }
          
          cres = stringSolver(counts, n);
          
          if (cres == null)
            return null;          
        }



      }finally {
        bCharMod26 = bOld;
      }
      debug("= shift counts");
      
      int m = 0; 
      int sum  = 0;
      for (int i = 0; i < n; i++) {
        m++;
        sum += cres[i].length();
        if (sum >= n)
          break;
      }
      
      String[] bres = stringSolver(bases, m);
      if (bres == null)
        return null;
      debug("= shift bases");

      String [] res = new String[n];
      
      int c = 0;
      solution:
      for (int i = 0; i < bres.length; i++) {
        res[c++] = bres[i];
        if (c >= n)
          break;
        for (int j = 1; j < cres[i].length(); j++) {
          int x = cres[i].charAt(j)-'A';
          if (bNegativeShifts)
            x = -x;
          res[c++] = shift(bres[i], x);
          if (c >= n)
            break solution;
        }
      }
      debug("< shiftpacker");
      return res;
    }finally {
      if (debug) dl--;
    }
  }

  /**
   * Does the actual replacing in patternSimplifier.
   */
  private static final String[] patternSimplifierCore(String[] q, int n, 
                                                      Object[] pairs, 
                                                      int pref) {
    debug("patternSimplifierCore");
    if (debug) dl++;
    try {
      String [] sq = new String[q.length];
      String [] dictionary = new String[pairs.length];
      int dc = -1;
      for (int i = 0; i < q.length; i++) {
        sq[i] = q[i];
      }
      for (int j = 0; j < pairs.length; j++) {
        Object []pair = (Object[])pairs[j];
        if (pref == -1 || pref == ((String)pair[0]).length()) {
          dictionary[++dc] = (String)pair[0];
        }
      }   
      for (int i = 0; i < q.length; i++)
        for (int j = 0; j <= dc; j++)
          sq[i] = replace(sq[i], dictionary[j], ""+(char)(1000+j));
      
      /*
      for (int i = 0; i < q.length; i++) {
        int len = sq[i].length();
        for (int j = 0; j < len; j++)
          System.out.print(" "+(int)sq[i].charAt(j));
        System.out.println();
        
      }
      */

      if (dc < 0)
        return null;
      boolean bOld = bCharMod26;
      bCharMod26 = false;
      try {
        String [] sres = stringSolver(sq, n);
        if (sres == null)
          return null;
        String [] res = new String[n];
        for (int i = 0; i < n; i++) {
          res[i] = sres[i];
          for (int j = dc; j >= 0; j--) {
            res[i] = replace(res[i], ""+(char)(1000+j), dictionary[j]);
          }

          //check for illeagal chars ~[0-9A-Z]
          int len = res[i].length();
          for (int j = 0; j < len; j++) {
            int c = res[i].charAt(j);
            if ( !((c >= '0' && c <= '9')
                   || (c >= 'A' && c <= 'Z')))
              return null;
          }
        }
        
        debug("< pattern simplifier core, pref "+pref);
        return res;

      }finally {
        bCharMod26 = bOld;
      }
    }
    finally {
      if (debug) dl--;
    }

  }

  /**
   * Finds most common substrings of the terms and replaces
   * them with a single character (unicode 1000+i) then solves
   * this queue with normal string solver.
   * 
   * ABYZYZYZYZYZ
   * ABABYZYZYZYZ
   * ABABABYZYZYZ
   * AB = a, YZ = b -> 
   * abbbbb, aabbbb, aaabbb
   */
  private static final String[] patternSimplifier(String [] q, int n) {
    debug("patternSimplifier");
    if (debug) dl++;
    try {
      Hashtable ht = new Hashtable();
      
      for (int i = 0; i < q.length; i++) {
        int len = q[i].length();
        for (int j = 2; j <= len; j++) {
          for (int k = (j>5) ? 5 : j ; k >= 2; k--) {
            String s = q[i].substring(j-k,j);
            Integer count = (Integer)ht.get(s);
            int c = 0;
            if (count == null) {
              c = 1;
            }
            else
              c = count.intValue()+1;
            ht.put(s, new Integer(c));
          }
        }
      }
      Vector v = new Vector();
      Enumeration e = ht.keys();
      while (e.hasMoreElements()) {
        String s = (String)e.nextElement();
        Integer count = (Integer)ht.get(s);
        v.addElement(new Object[] {s, count});
      }
      Object[] pairs = v.toArray();
      v.clear();
      v = null;
      ht.clear();
      ht = null;
      Arrays.sort(pairs, new Comparator() {
          public int compare(Object o1, Object o2) {
            Object[] a1 = (Object[])o1;
            Object[] a2 = (Object[])o2;
            
            int c = ((Integer)a1[1]).compareTo(a2[1]);
            if (c != 0)
              return -c;
            return ((String)a2[0]).length() - ((String)a1[0]).length();
          }
        });
      
      for (int i = 0; i < pairs.length; i++) {
        Object[] a = (Object[])pairs[i];
      }
        
      String [] res = patternSimplifierCore(q, n, pairs, -1);
      
      
      if (res == null)
        for (int i = 5; i >= 2; i--) {
          res = patternSimplifierCore(q, n, pairs, i);
          if (res != null)
            break;
        }
      
      if (res != null)
        debug("< pattern simplifier");
      return res;
    }finally {
      if (debug) dl--;
    }
  }

  /**
   * Even-odd split for strings.
   * A,1,AB,2,ABC,3 -> A,AB,ABC
   *                -> 1,2,3
   */
  private static final String[] splitEvenOdd(String  [] q, int n) {
    if (q.length < 6)
      return null;
    debug("even odd split");
    splitLevel ++;
    if (debug) dl++;
    try {
      int len = q.length;
      String [] even = new String[len/2+(len&0x1)];
      String [] odd  = new String[len/2];
      for (int i = 0; i < len; i++)
        if ( (i & 0x1) > 0) {
          odd[i/2] = q[i];
        }
        else {
          even[i/2] = q[i];
        }
      
      int en = n/2;
      int on = n/2;
      boolean bOddFirst = false;
      if ((len &0x1) > 0)
        bOddFirst = true;
      if ( (n &0x1) > 0) {
        if (bOddFirst)
          on++;
        else
          en++;
      }
            
      String [] eres = null;
      if (en > 0) {
        eres = stringSolver(even, en);
        if (eres == null)
          return null;
        debug("= even terms");
      }
      String [] ores = null;
      if (on > 0) {
        ores = stringSolver(odd, on);
        if (ores == null)
          return null;
        debug("= odd terms");
      }
      int c = 0;
      int oc= 0;
      int ec = 0;
      String [] res = new String[n];
      if (bOddFirst) {
        res[c++] = ores[oc++];
      }
      while(c < n) {
        res[c++] = eres[ec++];
        if (c < n)
          res[c++] = ores[oc++];
      }
      
      debug("< even odd split");
      return res;
    
      
    }finally {
      if (debug) dl--;
      splitLevel --;
    }
  }

  /**
   * Tries to solve a queue where each term is shifted
   * i*shiftFactor to right (i = number of term).
   * 1AAA
   * A2AA
   * AA3A
   * -> with shiftFactor = -1
   * -> 1AAA,2AAA,3AAA 
   */
  private static final String[] shiftedQueueSolver(String[] q, int n, 
                                                   int shiftFactor) {
    
    debug("shifted queue solver "+shiftFactor);
    if (debug) dl++;
    try {
      boolean bContinue = false;
      String[] sq = new String[q.length];
      for (int i = 0; i < q.length; i++) {
        int len = q[i].length();
        if (len > 1)
          bContinue = true;
        sq[i] = shift(q[i], shiftFactor*i);
      }
      if (!bContinue)
        return null;
      String [] sres = stringSolver(sq, n);
      if (sres == null)
        return null;
      
      String[] res = new String[n];
      
      for (int i = 0; i < n; i++) {
        res[i] = shift(sres[i], -(q.length+i)*shiftFactor);
      }
      debug("< shift factor "+shiftFactor);
      return res;
    }finally {
      if (debug) dl--;
    }
    
  }
  
  //TODO
  /**
   * Basically a number system without 0.
   * Base of the system is chosen to be the number
   * of different characters. Order of characters
   * is guessed to be the order in which they appear.
   * A,B,AA,AB,BA,BB,AAA
   * A = 1, B = 2, base = 2
   * -> 1,2,3,4,5,6,7...
   *
   * 0001, 0010, 0011
   * -> 15,16,17
   */
  private static final String[] charnumbers(String []q, int n) {
    if (q.length < 4)
      return null;
    debug("char numbers");
    if (debug) dl++;
    try {
      
      char[] digits = new char[4];
      int[] cs = new int[256];
      int base = 0;

      for (int i = 0; i < q.length; i++) {
        int len = q[i].length();
        for (int j = 0; j < len; j++) {
          char c = q[i].charAt(j);
          int x = c & 0xff;
          if (cs[x] == 0) {
            if (base >= digits.length)
              return null;
            digits[base++] = c;
            cs[x] = base;
          }
        }
      }
      if (base < 2 || q.length <= base)
        return null;
      
      int[] iq = new int[q.length];
      for (int i = 0; i < q.length; i++) {
        int len = q[i].length();
        int b = 1;
        int sum = 0;
        for (int j = len-1; j >= 0; j--, b*=base) {
          sum += cs[(int)q[i].charAt(j)]*b;
        }
        iq[i] = sum;
      }
  
      //for (int i = 0; i < iq.length; i++) {
      //  debug("::: "+iq[i]);
      //}

      int [] ires = simpleIntSolver(iq, n);
      if (ires == null)
        return null;

      String [] res = new String[n];
      for (int i = 0; i < n; i++) {
        int [] buffer = new int[maxtermlength+5];
        int x = ires[i];
        if (x < 1)
          return null;
        int bc = 0;
        while(x > 0) {
          buffer[bc++] = x%base;
          //  System.out.print(""+(x%base));
          x /= base;
          if (bc > maxtermlength+1)
            return null;
        }
        //System.out.println();
        for (int j = 0; j < bc-1; j++) {
          if (buffer[j] <= 0) {
            buffer[j+1]--;
            buffer[j] += base;
          }
          //System.out.print(""+buffer[j]);
        }
        //System.out.println("f");
        String s = "";
        for (int j = 0; j < bc; j++) {
          int ix =  buffer[j];
          if (ix <= 0)
            break;
          s = digits[ix-1]+s;
        }
        res[i] = s;
      }
      
      debug("< char numbers");
      return res;
    }finally {
      if (debug) dl--;
    }
    
  }
  
  /**
   * Test solution for illeagal characters
   */
  private static final String[] test(String[] q) {
    if (q == null)
      return null;
    if (stringSolverLevel > 1)
      return q;
    for (int i = 0; i < q.length; i++) {
      int len = q[i].length();
      if (len == 0) {
        debug("!! test failed, length 0");
        return null;
      }
      if (len > maxtermlength) {
        debug("!! test failed, too long " +len);
        return null;
      }
        
      for (int j = 0; j < len; j++) {
        char c = q[i].charAt(j);
        if (!((c >='0' && c <='9')
              ||(c >='A' && c <='Z'))) {
          debug("!! test failed, illeagal characters");
          return null;
        }
      }
        
    }
    return q;
  }
  
  /**
   * Basic solver function that tries to solve the queue
   * starting with algorithms that are less likely to return
   * wrong solutions. If these simple algorithms are not successful,
   * more powerfull and possibly more error prone algorithms are tried.
   * Most of these functions will do some reformatting to the queue 
   * and call stringSolver again recursively. 
   */
  private static final String[] stringSolver(String[] q, int n) {
    debug("string solver");
    if (debug) dl++;
    stringSolverLevel ++;
    try {
      
      String[] res = null;
      
      if (debug) {
        ASSERT(n > 0);
        ASSERT(n < maxterms*maxtermlength);
        ASSERT(q!=null);
        for (int i = 0; i < q.length; i++) {
          ASSERT(q[i] != null);
          ASSERT(!q[i].equals(""));
        }
      }
      
      res = repeater(q, n);
      if (res == null)
        res = loopFinder(q, n);
      
      //A, BB, BB, CCC, CCC, CCC
      if (res == null)
        res = test(termRepeat(q, n));
      
      //AA,ABAB,ABCABC
      if (res == null)
        res = test(patternRepeat(q, n));
      
      if (res == null && stringSolverLevel == 1)
        res = test(stringfib(q, n));
      
      if (res == null)
        res = test(stringRecursive(q, n));
      if (res == null)
        res = test(revStringRecursive(q, n));
      
      if (res == null)
        res = test(palindrome(q, n));
      
      if (res == null && stringSolverLevel == 1)
        res = test(numberPalindrome(q, n));
      
      if (res == null)
        res = test(shiftSolver(q, n));
      
      if (res == null)
        res = test(charAndIntSolver(q, n));

      if (res == null)
        res = test(charIntSplit(q,n));

      if (res == null && stringSolverLevel == 1)
        res = test(charnumbers(q, n));
      
      if (res == null)
        res = test(charwiseSplit(q, n));

      if (res == null)
        res = test(charPacker(q, n));
      
      
      if (res == null && stringSolverLevel == 1)
        for (int i = 1; i <= 3; i++) {
          res = test(shiftedQueueSolver(q, n, -i));
          if (res != null)
            break;
          res = test(shiftedQueueSolver(q, n, i));
          if (res != null)
            break;
        }
      
      if (res == null )
        res = test(flatlist(q, n));
      
      if (res == null)
        res = test(shiftPacker(q, n));

      
      if (res == null && stringSolverLevel == 1)
        res = test(splitEvenOdd(q, n));
      
      if (res == null && stringSolverLevel == 1)
        res = test(patternSimplifier(q, n));


      if (res != null)
        debug("< string solver");

      if (debug) {
        if (res != null)
          for (int i = 0; i < res.length; i++)
            ASSERT(res[i] != null);
      }
      return res;
    }
    finally {
      stringSolverLevel--;
      if (debug) dl--;
    }
  } 
  
  


  /**
   * Replaces all instances of 'search' in s with 'rep'.
   */
  private static final String replace(String s, String search, String rep) {
    StringBuffer sb = new StringBuffer();
    
    int ix = s.indexOf(search);
    int last = 0;
    int len = search.length();
    while (ix != -1) {
      sb.append( s.substring(last, ix) );
      sb.append( rep );
      last = ix+len;
      ix = s.indexOf(search, last);
    }
    sb.append(s.substring(last));
    return sb.toString();
  }

  /**
   * Reverses a string
   */
  private static final String reverse(String s) {
    StringBuffer sb = new StringBuffer(s);
    return sb.reverse().toString();
  }
  
  /**
   * Shift a string n character to right. Negative values
   * for n are allowed,
   */
  private static final String shift(String s, int n) {
    
    int len = s.length();
    if (len == 0)
      return s;
    if (n < 0)
      do { n += len; } while (n < 0);
    else
      n %= len;
    int x = len-n;
    char [] c = new char[len];
    for (int i = 0; i < x; i++)
      c[i+n] = s.charAt(i); 
    for (int i = x; i < len; i++)
      c[i-x] = s.charAt(i); 

    return new String(c);
  }

  /**
   * try to solver a queue where all terms are reversed
   */
  private static final String[] revSolver(String [] q, int n) {
    debug("revSolver");
    if (debug) dl++;
    try {
      String [] rq = new String[q.length];
      for (int i = 0; i < rq.length; i++)
        rq[i] = reverse(q[i]);

      String[] rres = stringSolver(rq, n);
      if (rres == null)
        return null;
      String [] res = new String[n];
      for (int i = 0; i < n; i++)
        res[i] = reverse( rres[i] );
      debug("< reverse solver");
      return res;
    }
    finally {
      if (debug) dl--;
    }
  }

  
  /**
   * Clips some terms out of the beginning or end of the queue
   * and tries to solve this shorter queue.
   */
  private static final String[] clipSolver(String [] q, int n, 
                                           int clipBegin, int clipEnd) {
    if (clipBegin == 0 && clipEnd == 0)
      return stringSolver(q, n);
    debug("clip solver "+clipBegin+" : "+clipEnd);
    
    if (clipBegin+clipEnd >= q.length)
      return null;
    
    String [] cs = new String[q.length-clipBegin-clipEnd];

    for (int i = 0; i < cs.length; i++) {
      cs[i] = q[clipBegin+i];
    }
    
    String [] sres = stringSolver(cs, n+clipEnd);
    if (sres == null)
      return null;
    if (clipEnd == 0) {
      debug("< clip solver "+clipBegin+" : "+clipEnd);
      return sres;
    }


    //check solution
    int x = q.length-clipEnd;
    for (int i = 0; i < clipEnd; i++) {
      if (!q[x+i].equals(sres[i]))
        return null;
    }

    String [] res = new String[n];
    for (int i = 0; i < n; i++)
      res[i] = sres[clipEnd+i];
    
    debug("< clip solver "+clipBegin+" : "+clipEnd);
    return res;
  }

  /**
   * Start point for the solver. Prints solution if one is found
   */
  private static final void solve(String [] q, int n) {
    diffLevel = 0;
    stringSolverLevel = 0;
    splitLevel = 0;
    stringRecLevel = 0;
    termRepeatLevel = 0;
    flatListLevel = 0;
    charwiseLevel = 0;
    intSolverLevel = 0;
    dl = 0;
    String [] res = null;
    
    for (int i = 0; i < q.length/2; i++) {
      res = clipSolver(q, n, 0, i);
      if (res != null)
        break;
    }
    
    if (res == null)
      res = revSolver(q, n);
    
    if (res == null) {
      solution:
      for (int i = 1; i < q.length/2; i++)
        for (int j = 0; j < q.length/2; j++) {
          if (2*(i+j)>q.length)
            break;
          res = clipSolver(q, n, i, j);
          if (res != null)
            break solution;
        }
    }

    if (res != null) {
      for (int i = 0; i < res.length; i++) {
        printSol(res[i]);
      }
      return;
    }    

    debug("No go");
  }
  

  
  private static int requiredElements = 0;
  /**
   * Read input file. Sets the number of required elements
   * and returns the queue
   */
  private static String [] readFile() {
    try {
      BufferedReader br = 
        new BufferedReader(new FileReader(name+".luk"));
      String sLine = br.readLine();
      StringTokenizer st = new StringTokenizer(sLine);
      int n = Integer.parseInt(st.nextToken());
      requiredElements = Integer.parseInt(st.nextToken());
      String [] q = new String[n];
      for (int i = 0; i < n; i++) {
        q[i] = br.readLine().trim();
      }
      br.close();
      return q;

    }catch( Exception ex ) {
      System.err.println("Exception occured while reading file.\n"
                         +"Incorrect file format?");
      ex.printStackTrace();
      System.exit(-1);
    }
    return null;
  }
  
  
  /**
   * Open the output file
   */
  private static void openFile() {
    try {
      fos = new FileOutputStream(new File(name+".kir"));
    }catch(IOException ex) {
      ex.printStackTrace();
      System.exit(-1);
    }
  }

  /**
   * Close the output file
   */
  private static void closeFile() {
    try {
      fos.close();
    }catch(IOException ex) {
      ex.printStackTrace();
      System.exit(-1);
    }
  }
  

  public static void main(String[] args) {
    String [] q = readFile();

    openFile();
    solve(q, requiredElements);

    closeFile();
  }

}
