#include <sstream>
#include <iostream>
#include "alphasolver.h"
#include "numericsolver.h" 
#include "solverutils.h"
using namespace std;

//-----------------------------------------------------------------------------
// Turn patterns with single characters into numbers and solve with
// numeric solver e.g.
// A,B,C,D
//
bool AlphaSolver::Solve()
//-----------------------------------------------------------------------------
    {
    vector<int> numbers;
    iAnswer.clear();
    if( iMaxIteration <= 0 )
        return false;
    if( iPattern.size() < 3 )
        return false;

    // Char to int
    int count = iPattern.size();
	for( int i = 0; i < count; i++ )
        {
        if( iPattern[i].size() != 1 )
            return false;
        if( ( iPattern[i][0] < 'A' || iPattern[i][0] > 'Z' ) &&
            ( iPattern[i][0] < '0' || iPattern[i][0] > '9' ) )
            return false;
        int chr = iPattern[i][0] - 'A';
        numbers.push_back( chr );
        }
    

    // Solve 
    NumericSolver* slv = new NumericSolver();
    slv->SetMaxIteration( iMaxIteration - 1 );
    slv->SetPattern( numbers, iRequired );
    if( !slv->Solve() )
        {
        // not solvable
        delete slv;
        return false;
        }

    // solved
    numbers.clear();
    slv->GetAnswer( numbers );

    iSolverStr = "(alph)" + slv->SolveString();
    delete slv;
    
    // int to char
    count = numbers.size();
    for( int i = 0; i < count; i++ )
        {
        char value = (char)( 'A' + numbers[i] );
        if( ( value < 'A' || value > 'Z' ) &&
            ( value < '0' || value > '9' ) )
            {
            // out of range! invalid answer.
            return false;
            }
        stringstream answ;
        answ << value;
        string str( answ.str() );
        iAnswer.push_back( str );
        }
    return true;
    }

