#include <sstream>
#include <iostream>
#include "slicesolver.h"
#include "patternsolver.h"
#include "solverutils.h"

using namespace std;

//-----------------------------------------------------------------------------
// Slice pattern into pieces and then try to solve
// eg. AABB,CCDD,EEFF ==> AA,BB,CC,DD,EE,FF
// eg. A,BC,DEF ==> A,B,C,D,E,F
//
bool SliceSolver::Solve()
//-----------------------------------------------------------------------------
{
    if( iMaxIteration <= 0 )
        return false;

    int count = iPattern.size();
    int minlen = count ? iPattern[0].length() : 0;
    for( int i = 1; i < count; i++ )
        {
        minlen = min( (int)(iPattern[i].length()), minlen );
        }

    for( int i = minlen; i > 0; i-- )
        {
        if( SliceSolve( i ) )
            return true;
        }

    return false;
}

//-----------------------------------------------------------------------------

bool SliceSolver::SliceSolve( int aSliceSize )
//-----------------------------------------------------------------------------
{
    vector<string> mergedPattern;
    vector<string> lengthPattern; // easier to solve as string
    int maxLength = 0;
    int count = iPattern.size();

    for( int i = 0; i < count; i++ )
        {
        // add length
        int len = iPattern[i].length();

        if( len % aSliceSize != 0 )
            return false; // pattern not dividable with this size

        maxLength = max( len, maxLength );
        len /= aSliceSize;
        stringstream lenstr;
        lenstr << (char)(len + 'A');
        lengthPattern.push_back( lenstr.str() );
        
        // add string
        for( int j = 0; j < len; j++ )
            {
            string chr( iPattern[i].substr( j * aSliceSize, aSliceSize ) );
            mergedPattern.push_back( chr );
            }
        }

    if( maxLength <= aSliceSize ) // no sense to slice if already in pieces
        return false;

    // First solve length array so that we know how many real answers we need
    PatternSolver* solver = new PatternSolver();
    solver->SetMaxIteration( iMaxIteration - 1 );
    solver->SetPattern( lengthPattern, iRequired );
    if( !solver->Solve() )
        {
        delete solver;
        return false;
        }

    solver->GetAnswer( lengthPattern );
    vector<int> lengthVect;
    int lensum = 0;
    for( int i = 0; i < lengthPattern.size(); i++ )
        {
        int len = lengthPattern[i][0] - 'A';
        lensum += len;
        lengthVect.push_back( len );
        }

    // Solve merged pattern
    solver->SetPattern( mergedPattern, lensum );
    if( !solver->Solve() )
        {
        delete solver;
        return false;
        }
    solver->GetAnswer( mergedPattern );

    // Reconstruct solved pattern
    count = lengthVect.size();
    int k = 0;
    for( int i = 0; i < count; i++ )
        {
        stringstream str;
        for( int j = 0; j < lengthVect[i]; j++ )
            {
            str << mergedPattern[k++];
            }
        iAnswer.push_back( str.str() );
        }

    stringstream str;
    str << "(slice:" << aSliceSize << ")";
    iSolverStr = str.str() + solver->SolveString();
    delete solver;
    return true;
}
