#include <iostream>
#include <sstream>
#include "trimsolver.h"
#include "patternsolver.h"

using namespace std;

//-----------------------------------------------------------------------------
// Trim patterns and then try to solve them. e.g.
// AAAB,AAABB,AAABBB => AB,AB,AB : 31,32,33(trim pattern)
//
bool TrimSolver::Solve()
//-----------------------------------------------------------------------------
{
    iAnswer.clear();
    if( iMaxIteration <= 0 )
        return false;

    vector<string> newPattern;
    vector<string> trimPatterns; // for easier solving, we use string here
    
    // Trim given pattern
    int count = iPattern.size();
    bool trimSuccess = false;
    for( int i = 0; i < count; i++ )
        {
        string str;
        string trim;
        TrimString( iPattern[i], str, trim );
        if( str != iPattern[i] )
            {
            trimSuccess = true;
            }
        newPattern.push_back( str );
        trimPatterns.push_back( trim );
        }

    // If trim didn't do anything, then there's no point solving this
    if( !trimSuccess )
        return false;

    // now we need to solve trimmed strings and the trim pattern
    PatternSolver* solver = new PatternSolver();
    solver->SetMaxIteration( iMaxIteration - 1 );
    solver->SetPattern( newPattern, iRequired );
    if( !solver->Solve() )
        {
        delete solver;
        return false;
        }
    iSolverStr = "(trim)[" + solver->SolveString() + " / ";
    solver->GetAnswer( newPattern );

    // Solve trim pattern
    solver->SetPattern( trimPatterns, iRequired );
    if( !solver->Solve() )
        {
        delete solver;
        return false;
        }
    iSolverStr += solver->SolveString() + "]";
    solver->GetAnswer( trimPatterns );

    // Expand patterns
    int cnt = newPattern.size();
    for( int i = 0; i < cnt; i++ )
        {
        string expstr;
        if( !ExpandString( newPattern[i], expstr, trimPatterns[i] ) )
            return false;
        iAnswer.push_back( expstr );
        }
    delete solver;
    return true;
}

//-----------------------------------------------------------------------------
void TrimSolver::TrimString( string& aSrc, string& aDest,
                             string& aTrimPattern )
//-----------------------------------------------------------------------------
{
    aTrimPattern.clear();
    
    int i = 0;
    int counter = 0;
    int len = aSrc.length();
    char lastChr;
    stringstream str;
    stringstream pattern;
    
    while( i < len )
        {
        if( counter == 0 )
            {
            counter++;
            lastChr = aSrc[i];
            }
        else if( lastChr == aSrc[i] ) // same char repeats
            {
            counter++;
            }
        else // new char -> reset counter
            {
            pattern << (char)(counter + 'A');
            str << lastChr;
            counter = 1;
            lastChr = aSrc[i];
            }
        i++;
        }
    pattern << (char)(counter + 'A');
    str << lastChr;

    aDest = str.str();
    aTrimPattern = pattern.str();
}

//-----------------------------------------------------------------------------
bool TrimSolver::ExpandString( string& aSrc, string& aDest,
                               string& aTrimPattern )
//-----------------------------------------------------------------------------
{
    if( aTrimPattern.length() != aSrc.length() )
        {
        //cout << "TrimSolver::ExpandString - size mismatch" << aTrimPattern
        //     << ":" << aSrc << endl;
        return false;
        }
    stringstream str;

    int count = aSrc.length();
    for( int i = 0; i < count; i++ )
        {
        for( int j = 0; j < aTrimPattern[i] - 'A'; j++)
            {
            str << aSrc[i];
            }
        }
    aDest = str.str();
    return true;
}

